<?php
/**
 * @package MachII_filters_ContinuationFilter
 */
/**
 * @package MachII_filters_ContinuationFilter
 * @author Sean Corfield
 * @author Alan Richmond (PHP port)
 */
/*
    I am an event filter that announces a continuation event.
    Example:
        An application protects an event with a login gate.
        When login is required, the event handler announces loginPage but
        needs a way to get back to that event handler after successful
        login. It sets continuationEvent in the event object and after a
        successful login, this filter is called to announce that event.
    Usage:
        <event-filter name="continue" type="MachII.filters.ContinuationFilter" />
        
        <event-handler event="doSomething" access="public">
            <event-arg name="continuationEvent" value="doSomethingElse" />
            <announce event="intermediate" />
        </event-handler>
        
        <event-handler event="intermediate" access="private">
            <filter name="continue" />
        </event-handler>
        
        <event-handler event="doSomethingElse" access="private">
            <!-- continuation filter gets us here -->
        </event-handler>
    Parameters:
        copyEventArgs                optional    should the filter copy the current event args
                                    default        true
        clearEventQueue                optional    should the filter clear the event queue
                                    default        false
        abortEvent                    optional    should the filter abort the current event
                                    default        true
        continuationEventArgName    optional    name of event argument containing continuation event name
                                    default        continuationEvent
    Event arguments:
        continuationEvent**            optional    event to announce
                                    default        do not announce an event
        ** the actual name of the argument is determined by continuationEventArgName
*/

class MachII_filters_ContinuationFilter extends MachII_framework_EventFilter
{
    function filterEvent(&$event, &$eventContext, &$paramArgs)
    {
        $copyEventArgs = (isset($paramArgs['copyEventArgs']))?
            $paramArgs['copyEventArgs']:
            $this->getParameter('copyEventArgs', true);
            
        $clearEventQueue = (isset($paramArgs['clearEventQueue']))?
            $paramArgs['clearEventQueue']:
            $this->getParameter('clearEventQueue', false);
            
        $abortEvent = (isset($paramArgs['abortEvent']))?
            $paramArgs['abortEvent']:
            $this->getParameter('abortEvent', true);
            
        $continuationEventArgName = (isset($paramArgs['continuationEventArgName']))?
            $paramArgs['continuationEventArgName']:
            $this->getParameter('continuationEventArgName', 'continuationEvent');
            
        if ($event->isArgDefined($continuationEventArgName)) {
            $continuationEvent = $event->getArg($continuationEventArgName);
            $event->removeArg($continuationEventArgName);
            if ($copyEventArgs) {
                $continuationArgs =& $event->getArgs();
            } else {
                $continuationArgs = array();
            }
            if ($clearEventQueue) {
                $eventContext->clearEventQueue();
            }
            $eventContext->announceEvent($continuationEvent, $continuationArgs);
            
            return !$abortEvent;
        }
        
        return true;
    }
}

?>